@@ -12,9 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <lrx_processor.h>
@@ -493,6 +491,7 @@ LRXProcessor::process(FILE *input, FILE *output)
   vector<State> alive_states_clean ;
   vector<State> alive_states = alive_states_clean ;
   alive_states.push_back(*initial_state);
+  vector<State> new_states;
 
   int last_final = -1; // check what we actually use this for
 
@@ -561,7 +560,7 @@ LRXProcessor::process(FILE *input, FILE *output)
         fwprintf(stderr, L"[POS] %d: [sl %d ; tl %d ; bl %d]\n", pos, sl[pos].size(), tl[pos].size(), blanks[pos].size());
       }
 
-      vector<State> new_state; // alive_states_new 
+      new_states.clear(); // alive_states_new 
       pair<double, vector<State> > new_best_cover;
       new_best_cover.first = -numeric_limits<int>::max();
 
@@ -578,7 +577,7 @@ LRXProcessor::process(FILE *input, FILE *output)
         if(s.size() > 0) // If the current state has outgoing transitions, 
                          // add it to the new alive states
         {
-          new_state.push_back(s);
+          new_states.push_back(s);
         }
         s.step(alphabet(L"<$>"));
 
@@ -660,7 +659,7 @@ LRXProcessor::process(FILE *input, FILE *output)
         }
       }
  
-      alive_states = new_state;
+      alive_states.swap(new_states);
       alive_states.push_back(*initial_state);
 
       if(debugMode)
@@ -717,7 +716,7 @@ LRXProcessor::process(FILE *input, FILE *output)
         fwprintf(stderr, L"outOfWord = false\n");
       }
 
-      vector<State> new_state;
+      new_states.clear();
       wstring res = L"";
       for(vector<State>::const_iterator it = alive_states.begin(); it != alive_states.end(); it++)
       {
@@ -742,14 +741,14 @@ LRXProcessor::process(FILE *input, FILE *output)
         }
         if(s.size() > 0) // If the current state has outgoing transitions, add it to the new alive states
         {
-          new_state.push_back(s);
+          new_states.push_back(s);
         }
       }
       if(debugMode)
       {
-        fwprintf(stderr, L"new_state: %d\n", new_state.size());
+        fwprintf(stderr, L"new_states: %d\n", new_states.size());
       }
-      alive_states = new_state;
+      alive_states.swap(new_states);
       alive_states.push_back(*initial_state);
 
     }
@@ -799,6 +798,7 @@ LRXProcessor::processME(FILE *input, FILE *output)
   vector<State> alive_states_clean ;
   vector<State> alive_states = alive_states_clean ;
   alive_states.push_back(*initial_state);
+  vector<State> new_states;
 
   while(!feof(input))
   {
@@ -806,6 +806,12 @@ LRXProcessor::processME(FILE *input, FILE *output)
 
     if(nullFlush && val == L'\0') 
     {
+      processFlushME(output, sl, tl, blanks, scores);
+      pos = 0;
+      tl.clear();
+      sl.clear();
+      blanks.clear();
+
       fputwc_unlocked(val, output);
       fflush(output);
       continue;
@@ -854,7 +860,7 @@ LRXProcessor::processME(FILE *input, FILE *output)
       {
         fwprintf(stderr, L"[POS] %d: [sl %d ; tl %d ; bl %d]: %S\n", pos, sl[pos].size(), tl[pos].size(), blanks[pos].size(), sl[pos].c_str());
       }
-      vector<State> new_state; // alive_states_new 
+      new_states.clear(); // alive_states_new 
 
       // \forall s \in A
       set<wstring> seen_ids;
@@ -868,7 +874,7 @@ LRXProcessor::processME(FILE *input, FILE *output)
         if(s.size() > 0) // If the current state has outgoing transitions, 
                          // add it to the new alive states
         {
-          new_state.push_back(s);
+          new_states.push_back(s);
         }
         s.step(alphabet(L"<$>"));
 
@@ -929,7 +935,7 @@ LRXProcessor::processME(FILE *input, FILE *output)
           }
         }
       }
-      alive_states = new_state;
+      alive_states.swap(new_states);
       alive_states.push_back(*initial_state);
 
       if(debugMode)
@@ -1078,7 +1084,7 @@ LRXProcessor::processME(FILE *input, FILE *output)
         fwprintf(stderr, L"outOfWord = false\n");
       }
 
-      vector<State> new_state;
+      new_states.clear();
       wstring res = L"";
       for(vector<State>::const_iterator it = alive_states.begin(); it != alive_states.end(); it++)
       {
@@ -1103,14 +1109,14 @@ LRXProcessor::processME(FILE *input, FILE *output)
         }
         if(s.size() > 0) // If the current state has outgoing transitions, add it to the new alive states
         {
-          new_state.push_back(s);
+          new_states.push_back(s);
         }
       }
       if(debugMode)
       {
-        fwprintf(stderr, L"new_state: %d\n", new_state.size());
+        fwprintf(stderr, L"new_states: %d\n", new_states.size());
       }
-      alive_states = new_state;
+      alive_states.swap(new_states);
       alive_states.push_back(*initial_state);
 
     }
@@ -1142,88 +1148,96 @@ LRXProcessor::processME(FILE *input, FILE *output)
 
   }
 
-  // Here we actually apply the rules that we've matched
+  processFlushME(output, sl, tl, blanks, scores);
+}
+
+void
+LRXProcessor::processFlushME(FILE *output,
+                             map<int, wstring > &sl,
+                             map<int, vector<wstring> > &tl,
+                             map<int, wstring > &blanks,
+                             map<int, map<wstring, double> > &scores) {
 
   unsigned int spos = 0;
   for(spos = 0; spos <= pos; spos++)
   {
     if(sl[spos] == L"")
-          {
-            continue;
-          }
+    {
+      continue;
+    }
 
-          fwprintf(stdout, L"%S^%S/", blanks[spos].c_str(), sl[spos].c_str());
+    fwprintf(output, L"%S^%S/", blanks[spos].c_str(), sl[spos].c_str());
 
-          vector<wstring>::iterator ti;
-          vector<wstring>::iterator penum = tl[spos].end(); penum--;
+    vector<wstring>::iterator ti;
+    vector<wstring>::iterator penum = tl[spos].end(); penum--;
 
-          if(tl[spos].size() > 1)
-          {
-            //--
-            double l_max = 0.0;
-            wstring ti_max;
-            for(ti = tl[spos].begin(); ti != tl[spos].end(); ti++)
-            {
+    if(tl[spos].size() > 1)
+    {
+      //--
+      double l_max = 0.0;
+      wstring ti_max;
+      for(ti = tl[spos].begin(); ti != tl[spos].end(); ti++)
+      {
 
-                map<wstring, double>::iterator si;
-                for(si = scores[spos].begin(); si != scores[spos].end(); si++) 
-                {
-                  if(debugMode) 
-                  {
-                    fwprintf(stderr, L">>> %d -> %S -> %.5f\n", spos, si->first.c_str(), si->second);
-                  }
-                  bool matched = false;
-                  matched = recognisePattern(*ti, si->first);
-                  if(si->second > l_max && matched) 
-                  { 
-                    l_max = si->second;
-                    ti_max = *ti;
-                  }
-                }
-            }
-
-            if(l_max > 0.0)  // If we actually got a winner
-            {
-              if(traceMode || debugMode)
-              {
-                //fwprintf(stderr, L"%d: %d: %S -> %S (%d)\n", lineno, spos, sl[spos].c_str(), ti->c_str(), scores[spos].size());
-                //fwprintf(stderr, L"MAX: %.5f = %S\n", l_max, ti_max.c_str());
-                fwprintf(stderr, L"%d:SELECT:%.5f:%S:%S\n", lineno, l_max, sl[spos].c_str(), ti_max.c_str());
-              }
-              fwprintf(stdout, L"%S", ti_max.c_str());
-            }
-            else
-            {
-              for(ti = tl[spos].begin(); ti != tl[spos].end(); ti++)
-              {
-                fwprintf(stdout, L"%S", ti->c_str());
-                if(ti != penum)
-                {
-                  fwprintf(stdout, L"/");
-                }
-              }
-            }
-          }
-          else
-          {
-            for(ti = tl[spos].begin(); ti != tl[spos].end(); ti++)
-            {
-              fwprintf(stdout, L"%S", ti->c_str());
-              if(ti != penum)
-              {
-                fwprintf(stdout, L"/");
-              }
-            }
-          }
-
-          fwprintf(stdout, L"$");
+        map<wstring, double>::iterator si;
+        for(si = scores[spos].begin(); si != scores[spos].end(); si++)
+        {
           if(debugMode)
           {
-            fwprintf(stdout, L"%d", spos);
+            fwprintf(stderr, L">>> %d -> %S -> %.5f\n", spos, si->first.c_str(), si->second);
           }
+          bool matched = false;
+          matched = recognisePattern(*ti, si->first);
+          if(si->second > l_max && matched)
+          {
+            l_max = si->second;
+            ti_max = *ti;
+          }
+        }
+      }
+
+      if(l_max > 0.0)  // If we actually got a winner
+      {
+        if(traceMode || debugMode)
+        {
+          //fwprintf(stderr, L"%d: %d: %S -> %S (%d)\n", lineno, spos, sl[spos].c_str(), ti->c_str(), scores[spos].size());
+          //fwprintf(stderr, L"MAX: %.5f = %S\n", l_max, ti_max.c_str());
+          fwprintf(stderr, L"%d:SELECT:%.5f:%S:%S\n", lineno, l_max, sl[spos].c_str(), ti_max.c_str());
+        }
+        fwprintf(output, L"%S", ti_max.c_str());
+      }
+      else
+      {
+        for(ti = tl[spos].begin(); ti != tl[spos].end(); ti++)
+        {
+          fwprintf(output, L"%S", ti->c_str());
+          if(ti != penum)
+          {
+            fwprintf(output, L"/");
+          }
+        }
+      }
+    }
+    else
+    {
+      for(ti = tl[spos].begin(); ti != tl[spos].end(); ti++)
+      {
+        fwprintf(output, L"%S", ti->c_str());
+        if(ti != penum)
+        {
+          fwprintf(output, L"/");
+        }
+      }
+    }
+
+    fwprintf(output, L"$");
+    if(debugMode)
+    {
+      fwprintf(output, L"%d", spos);
+    }
 
 
   }
 
-  fwprintf(stdout, L"%S", blanks[pos].c_str());
+  fwprintf(output, L"%S", blanks[pos].c_str());
 }
